{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module Show where {

import qualified List;
import qualified Nat;
import qualified Integer;
import qualified Util;

sEmpty :: [Char];
sEmpty = [];

class Sshow a where {
  sshow :: a -> [Char];
};

tabC :: Char;
tabC = '\^I';

tab :: [Char];
tab = [Show.tabC];

newlineC :: Char;
newlineC = '\^J';

newline :: [Char];
newline = [Show.newlineC];

sshowa :: forall c d. (Show.Sshow c, Show.Sshow d) => (c, d) -> [Char];
sshowa (x, y) = List.concata [Show.sshow x, " -> ", Show.sshow y];

instance (Show.Sshow a, Show.Sshow b) => Show.Sshow (a, b) where {
  sshow = Show.sshowa;
};

digit2string :: Nat.Nat -> [Char];
digit2string n =
  (if Nat.eq_nat n Nat.Zero_nat then "0"
    else (if Nat.eq_nat n Nat.one_nat then "1"
           else (if Nat.eq_nat n (Integer.nat 2) then "2"
                  else (if Nat.eq_nat n (Integer.nat 3) then "3"
                         else (if Nat.eq_nat n (Integer.nat 4) then "4"
                                else (if Nat.eq_nat n (Integer.nat 5) then "5"
                                       else (if Nat.eq_nat n (Integer.nat 6)
      then "6"
      else (if Nat.eq_nat n (Integer.nat 7) then "7"
             else (if Nat.eq_nat n (Integer.nat 8) then "8"
                    else (if Nat.eq_nat n (Integer.nat 9) then "9"
                           else "not a digit"))))))))));

sshow_nat :: Nat.Nat -> [Char];
sshow_nat n =
  (if Nat.less_nat n (Integer.nat 10) then Show.digit2string n
    else List.append (Show.sshow_nat (Nat.div_nat n (Integer.nat 10)))
           (Show.digit2string (Nat.mod_nat n (Integer.nat 10))));

showLista :: [[Char]] -> [Char];
showLista xs = List.concata (List.mapa (\ x -> List.append x Show.newline) xs);

sshow_char :: Char -> [Char];
sshow_char x = [x];

instance Show.Sshow Char where {
  sshow = Show.sshow_char;
};

sshow_list :: forall a. (Show.Sshow a) => [a] -> [Char];
sshow_list xs = List.concata (List.mapa Show.sshow xs);

instance (Show.Sshow a) => Show.Sshow [a] where {
  sshow = Show.sshow_list;
};

sshow_cint :: Util.Cint -> [Char];
sshow_cint (Util.Neg n) =
  (if Nat.eq_nat n Nat.Zero_nat then Show.sshow_nat n
    else List.append "-" (Show.sshow_nat n));
sshow_cint (Util.Pos n) = Show.sshow_nat n;

sshow_int :: Integer -> [Char];
sshow_int i = Show.sshow_cint (Util.intToCint i);

}
