{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module Util where {

import List;
import Nat;
import Product_Type;
import HOL;

data Eithera a b = Righta b | Lefta a;

either_case :: forall a t b. (a -> t) -> (b -> t) -> Eithera a b -> t;
either_case f1 f2 (Lefta a) = f1 a;
either_case f1 f2 (Righta b) = f2 b;

data Cint = Pos Nat | Neg Nat;

testPos :: forall a. (a -> Bool) -> [a] -> Bool;
testPos f (x : xs) = f x || testPos f xs;
testPos uu [] = False;

unionI :: forall a. (Eq a) => [a] -> [a] -> [a];
unionI (x : xs) ys =
  let {
    zs = unionI xs ys;
  } in (if member x zs then zs else x : zs);
unionI [] ys = ys;

intToPos :: Integer -> Nat;
intToPos i = (if i <= 0 then Zero_nat else Suc (intToPos (i - 1)));

intToCint :: Integer -> Cint;
intToCint x = (if x < 0 then Neg (intToPos (negate x)) else Pos (intToPos x));

cint_case :: forall t. (Nat -> t) -> (Nat -> t) -> Cint -> t;
cint_case f1 f2 (Neg nat) = f1 nat;
cint_case f1 f2 (Pos nat) = f2 nat;

computeSplit :: forall a. (a -> Bool) -> [a] -> ([a], [a]);
computeSplit f (x : xs) =
  let {
    a = computeSplit f xs;
    (l, r) = a;
  } in (if f x then (x : l, r) else (l, x : r));
computeSplit uu [] = ([], []);

}
