(*
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory Show imports Util Main (* Code_Char_chr *)
begin

types sstring = string

class "show" = type + 
  fixes "show" :: "'a \<Rightarrow> string"

definition unquote :: "sstring \<Rightarrow> sstring"
  where "unquote x \<equiv> filter (\<lambda> c. \<not> c = CHR ''\"'') x"
definition showF :: "'a :: show \<Rightarrow> sstring"
  where "showF x \<equiv> unquote (show x)"

class sshow = type +
  fixes sshow :: "'a \<Rightarrow> sstring"


abbreviation sAppend :: "sstring \<Rightarrow> sstring \<Rightarrow> sstring" (infixl "+++" 10)
  where "sAppend s t \<equiv> (s @ t)"


abbreviation sConcat :: "sstring list \<Rightarrow> sstring" where "sConcat \<equiv> concat"


definition sEmpty :: "sstring" where "sEmpty = []"


instantiation bool  :: sshow
begin
fun sshow_bool where "sshow_bool True = ''True''" | "sshow_bool False = ''False''"
instance by intro_classes
end

instantiation char  :: sshow
begin
fun sshow_char where "sshow_char x = ([x] :: string)"
instance by intro_classes
end

instantiation list :: ("sshow") "sshow"
begin
fun sshow_list where "sshow xs = sConcat (map sshow xs)"
instance by intro_classes
end

definition newlineC :: char where "newlineC \<equiv> Char Nibble0 NibbleA"
definition tabC :: char where "tabC \<equiv> Char Nibble0 Nibble9"
definition tab :: string where "tab \<equiv> [tabC]"
definition newline :: string where "newline \<equiv> [newlineC]"
 

instantiation * :: (sshow,sshow) sshow
begin
fun sshow_prod where "sshow_prod (x,y) = (concat [sshow x, '' -> '', sshow y])"
instance by intro_classes
end




fun showList  :: "sstring list \<Rightarrow> string"
 where "showList xs = concat (map (\<lambda> x. x @ newline) xs)"


instantiation nat :: sshow
begin
fun digit2string :: "nat \<Rightarrow> string"
where "digit2string n = 
   (if n = 0 then ''0'' else
   (if n = 1 then ''1'' else
   (if n = 2 then ''2'' else
   (if n = 3 then ''3'' else
   (if n = 4 then ''4'' else
   (if n = 5 then ''5'' else
   (if n = 6 then ''6'' else
   (if n = 7 then ''7'' else
   (if n = 8 then ''8'' else
   (if n = 9 then ''9'' else ''not a digit''))))))))))"
  
fun sshow_nat :: "nat \<Rightarrow> string" 
where "sshow_nat n = (if n < 10 then digit2string n else sshow_nat (n div 10) @ digit2string (n mod 10))"
instance by intro_classes
end

instantiation cint :: sshow
begin
fun sshow_cint where
  "sshow_cint (Pos n) = sshow n"
| "sshow_cint (Neg n) = (if n = 0 then sshow n else (''-'' @ sshow n))"
instance ..
end

instantiation int :: sshow
begin
fun sshow_int where "sshow_int i = sshow (intToCint i)"
instance .. 
end

end


